/*
 * 2D Game Engine 
 * Render.h: 
 * Based on pikuma.com 2D game engine in C++ and Lua course
 * Copyright (c) 2021 986-Studio. All rights reserved.
 *
 * Created by Manoël Trapier on 12/02/2021.
 */

#ifndef GAMEENGINE_RENDER_H
#define GAMEENGINE_RENDER_H

#include <SDL.h>

#include <ECS.h>
#include <Components/Transform.h>
#include <Components/Sprite.h>

class RenderSystem: public System
{
public:
    RenderSystem()
    {
        this->requireComponent<TransformComponent>();
        this->requireComponent<SpriteComponent>();
    }

    void update(SDL_Renderer *renderer)
    {
        for (auto entity: this->getSystemEntities())
        {
            const auto tranform = entity.getComponent<TransformComponent>();
            const auto sprite = entity.getComponent<SpriteComponent>();

            SDL_Rect objRect =
            {
                static_cast<int>(tranform.position.x), static_cast<int>(tranform.position.y),
                static_cast<int>(sprite.width), static_cast<int>(sprite.height)
            };

            SDL_SetRenderDrawColor(renderer, 255, 127, 64, 255);
            SDL_RenderFillRect(renderer, &objRect);

        }
    }
};

#endif /* GAMEENGINE_RENDER_H */
